@@ -36,6 +36,8 @@
 #include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <eel/eel-input-event-box.h>
 
 #include <bonobo/bonobo-i18n.h>
 
@@ -47,6 +49,8 @@
 #include "gedit2.h"
 #include "gedit-print.h"
 #include "recent-files/egg-recent-util.h"
+#include "gedit-encodings.h"
+#include "gedit-tooltips.h"
 
 struct _GeditMDIChildPrivate
 {
@@ -56,6 +60,7 @@ struct _GeditMDIChildPrivate
 enum {
 	STATE_CHANGED,
 	UNDO_REDO_STATE_CHANGED,
+	FIND_STATE_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -75,6 +80,9 @@ static void gedit_mdi_child_document_readonly_changed_handler (GeditDocument *do
 static void gedit_mdi_child_document_can_undo_redo_handler (GeditDocument *document, 
 						gboolean can, GeditMDIChild* child);
 
+static void gedit_mdi_child_document_can_find_again_handler (GeditDocument *document, 
+							     GeditMDIChild* child);
+
 static gchar* gedit_mdi_child_get_config_string (BonoboMDIChild *child, gpointer data);
 
 
@@ -127,6 +135,7 @@ gedit_mdi_child_class_init (GeditMDIChildClass *klass)
 
 	klass->state_changed 		= gedit_mdi_child_real_state_changed;
 	klass->undo_redo_state_changed  = NULL;
+	klass->find_state_changed	= NULL;
   		
 	mdi_child_signals [STATE_CHANGED] =
 		g_signal_new ("state_changed",
@@ -148,6 +157,16 @@ gedit_mdi_child_class_init (GeditMDIChildClass *klass)
 			      G_TYPE_NONE, 
 			      0);
                     
+	mdi_child_signals [FIND_STATE_CHANGED] =
+		g_signal_new ("find_state_changed",
+			      G_OBJECT_CLASS_TYPE (gobject_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (GeditMDIChildClass, find_state_changed),
+			      NULL, NULL,
+			      gedit_marshal_VOID__VOID,
+			      G_TYPE_NONE, 
+			      0);
+                    
 	BONOBO_MDI_CHILD_CLASS (klass)->create_view = 
 		(BonoboMDIChildViewCreator)(gedit_mdi_child_create_view);
 
@@ -262,6 +281,15 @@ gedit_mdi_child_document_can_undo_redo_handler (GeditDocument *document, gboolea
 	g_signal_emit (G_OBJECT (child), mdi_child_signals [UNDO_REDO_STATE_CHANGED], 0);
 }
 
+static void 
+gedit_mdi_child_document_can_find_again_handler (GeditDocument *document, GeditMDIChild* child)
+{
+	gedit_debug (DEBUG_MDI, "");
+	g_return_if_fail (child->document == document);
+
+	g_signal_emit (G_OBJECT (child), mdi_child_signals [FIND_STATE_CHANGED], 0);
+}
+
 
 static void
 gedit_mdi_child_connect_signals (GeditMDIChild *child)
@@ -281,6 +309,9 @@ gedit_mdi_child_connect_signals (GeditMDIChild *child)
 	g_signal_connect (G_OBJECT (child->document), "can_redo",
 			  G_CALLBACK (gedit_mdi_child_document_can_undo_redo_handler), 
 			  child);
+	g_signal_connect (G_OBJECT (child->document), "can_find_again",
+			  G_CALLBACK (gedit_mdi_child_document_can_find_again_handler), 
+			  child);
 }
 
 /**
@@ -316,14 +347,14 @@ gedit_mdi_child_new (void)
 }
 
 GeditMDIChild*
-gedit_mdi_child_new_with_uri (const gchar *uri, GError **error)
+gedit_mdi_child_new_with_uri (const gchar *uri, const GeditEncoding *encoding, GError **error)
 {
 	GeditMDIChild *child;
 	GeditDocument* doc;
 	
 	gedit_debug (DEBUG_MDI, "");
 
-	doc = gedit_document_new_with_uri (uri, error);
+	doc = gedit_document_new_with_uri (uri, encoding, error);
 
 	if (doc == NULL)
 	{
@@ -427,7 +458,7 @@ gedit_mdi_child_tab_save_as_clicked (GtkWidget *button,  GtkWidget *view)
 static void
 gedit_mdi_child_tab_print_clicked (GtkWidget *button,  GtkWidget *view)
 {
-	BonoboMDIChild *child;
+	GeditDocument *doc;
 	
 	gedit_debug (DEBUG_MDI, "");
 
@@ -435,10 +466,10 @@ gedit_mdi_child_tab_print_clicked (GtkWidget *button,  GtkWidget *view)
 
 	bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
 	
-	child = bonobo_mdi_child_get_from_view (view);
-	g_return_if_fail (GEDIT_IS_MDI_CHILD (child));
-	
-	gedit_print (GEDIT_MDI_CHILD (child));
+	doc = gedit_view_get_document (GEDIT_VIEW (view));
+	g_return_if_fail (doc != NULL);
+		
+	gedit_print (doc);
 }
 
 static void
@@ -567,6 +598,8 @@ set_tab_icon (GtkWidget *image, BonoboMDIChild *child)
 	gchar *mime_type = NULL;
 	int icon_size;
 
+	gedit_debug (DEBUG_MDI, "");
+
 	g_return_val_if_fail (GTK_IS_IMAGE (image), NULL);
 	g_return_val_if_fail (GEDIT_IS_MDI_CHILD (child), NULL);
 
@@ -577,7 +610,7 @@ set_tab_icon (GtkWidget *image, BonoboMDIChild *child)
 
 	g_return_val_if_fail (theme != NULL, NULL);
 
-	raw_uri = gedit_document_get_uri (GEDIT_MDI_CHILD (child)->document);	
+	raw_uri = gedit_document_get_raw_uri (GEDIT_MDI_CHILD (child)->document);	
 	
 	if (raw_uri != NULL)
 		mime_type = gnome_vfs_get_mime_type (raw_uri);
@@ -601,16 +634,76 @@ set_tab_icon (GtkWidget *image, BonoboMDIChild *child)
 
 	return image;	
 }
+
+static void
+set_tooltip (GeditTooltips *tooltips, GtkWidget *widget, BonoboMDIChild *child)
+{
+	gchar *tip;
+	gchar *uri;
+	gchar *raw_uri;
+	gchar *mime_type = NULL;
+	const gchar *mime_description = NULL;
+	gchar *mime_full_description; 
+	gchar *encoding;
+	const GeditEncoding *enc;
 	
+	gedit_debug (DEBUG_MDI, "");
+
+	uri = gedit_document_get_uri (GEDIT_MDI_CHILD (child)->document);
+	g_return_if_fail (uri != NULL);
+
+	raw_uri = gedit_document_get_raw_uri (GEDIT_MDI_CHILD (child)->document);	
+	
+	if (raw_uri != NULL)
+		mime_type = gnome_vfs_get_mime_type (raw_uri);
+
+	if (mime_type != NULL)
+		mime_description = gnome_vfs_mime_get_description (mime_type);
+
+	if (mime_description == NULL)
+		mime_full_description = g_strdup (_("Unknown"));
+	else
+		mime_full_description = g_strdup_printf ("%s (%s)", 
+				mime_description, mime_type);
+		
+	enc = gedit_document_get_encoding (GEDIT_MDI_CHILD (child)->document);
+
+	if (enc == NULL)
+		encoding = g_strdup (_("Unicode (UTF-8)"));
+	else
+		encoding = gedit_encoding_to_string (enc);
+		
+	tip = g_strdup_printf ("<b>Name:</b> %s\n\n"
+			       "<b>MIME Type:</b> %s\n"
+			       "<b>Encoding:</b> %s", 
+			       uri, mime_full_description, encoding);
+
+	gedit_tooltips_set_tip (tooltips, widget, tip, NULL);
+	g_free (tip);
+
+	g_free (uri);
+	g_free (raw_uri);
+	g_free (mime_type);
+	g_free (encoding);
+	g_free (mime_full_description);
+}
+
+/**
+ * gedit_mdi_child_set_label:
+ * 
+ * Creates a tab label for the selected #BonoboMDIChild or 
+ * updates the incoming tab label.
+ *
+ * Return value: a tab label
+ **/
 static GtkWidget *
 gedit_mdi_child_set_label (BonoboMDIChild *child, GtkWidget *view,  GtkWidget *old_hbox,
                            gpointer data)
 {
 	GtkWidget *ret;
 	gchar *name;
-	gchar *uri;
 
-	static GtkTooltips *tooltips = NULL;
+	static GeditTooltips *tooltips = NULL;
 	
 	gedit_debug (DEBUG_MDI, "");
 
@@ -619,88 +712,106 @@ gedit_mdi_child_set_label (BonoboMDIChild *child, GtkWidget *view,  GtkWidget *o
 	g_return_val_if_fail (GEDIT_IS_VIEW (view), NULL);
 		
 	if (tooltips == NULL)
-		tooltips = gtk_tooltips_new ();
+	{
+		tooltips = gedit_tooltips_new ();
+	}
 	
+	/* setup the name for the tab title */
 	name = bonobo_mdi_child_get_name (child);
-
- 	uri = gedit_document_get_uri (GEDIT_MDI_CHILD (child)->document);
-	g_return_val_if_fail (uri != NULL, NULL);
 		
 	if (old_hbox != NULL) 
 	{
-		GtkWidget *label;
-		GtkWidget *image;
+		GtkWidget *image, *label, *event_box;
 
+		/* fetch the label/image/event_box */
 		label = g_object_get_data (G_OBJECT (old_hbox),	"label");
 		image = g_object_get_data (G_OBJECT (old_hbox),	"image");
+		event_box = g_object_get_data (G_OBJECT(old_hbox),"event_box");
 	
+		/* update the label/image/tooltip */
 		gtk_label_set_text (GTK_LABEL (label), name);
 		set_tab_icon (image, child);
 
+		set_tooltip (tooltips, event_box, child);
+		
 		ret = old_hbox;
 	} 
 	else 
 	{
-		GtkWidget *event_box;	
-		GtkSettings *settings;
-		GtkWidget *hbox;
-		GtkWidget *label;
-		GtkWidget *button;
-		GtkWidget *image;
+		GtkWidget *image, *label, *button;
+		GtkWidget *event_box, *event_hbox, *hbox;
 		GtkWidget *popup_menu;
+		GtkSettings *settings;
 		gint w, h;
-
+	
+		/* fetch the size of an icon */
 		settings = gtk_widget_get_settings (view);
-
 		gtk_icon_size_lookup_for_settings (settings,
 						   GTK_ICON_SIZE_MENU,
 						   &w, &h);
 
-		label = gtk_label_new (name);
-		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		/* create our layout/event boxes */
+		event_box = eel_input_event_box_new();
+		event_hbox = gtk_hbox_new (FALSE, 0);
+		hbox = gtk_hbox_new (FALSE, 0);
 
+		/* setup the close button */
 		button = gtk_button_new ();
-				
+		gtk_button_set_relief ( GTK_BUTTON (button),
+					GTK_RELIEF_NONE);
 		g_signal_connect (button, "clicked",
 				  G_CALLBACK (gedit_mdi_child_tab_close_clicked),
 				  view);
 		
-		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+		/* setup the close button's image */
 		image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 		gtk_widget_set_size_request (button, w + 2, h + 2);
-
 		gtk_container_add (GTK_CONTAINER (button), image);
 		
+		/* setup the document image */
 		image = gtk_image_new ();
 		set_tab_icon (image, child);
-		
 		gtk_widget_show (image);
 
-		hbox = gtk_hbox_new (FALSE, 2);
+		/* setup the tab title */
+		label = gtk_label_new (name);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_misc_set_padding (GTK_MISC (label), 4, 0);
 
-		event_box = gtk_event_box_new ();
+		/* pack the elements */
+		gtk_box_pack_start (GTK_BOX (event_hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (event_hbox), label, TRUE, FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (event_box), event_hbox);
+
+		/* setup the data hierarchy */
+		g_object_set_data (G_OBJECT (hbox), "label", label);
+		g_object_set_data (G_OBJECT (hbox), "image", image);
+		g_object_set_data (G_OBJECT (hbox), "event_box", event_box);
 			
-		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-		gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);
+		/* pack our top-level layout box */
+		gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 		
-		g_object_set_data (G_OBJECT (event_box), "label", label);
-		g_object_set_data (G_OBJECT (event_box), "image", image);
-		
+		/* setup popup menu */
 		popup_menu = create_popup_menu (child, view);
 		
+		/* un/set widget flags. otherwise  menu_attach() iterates
+		 * upwards (hierarchically) and rejects our event_box. 
+		 */
+		GTK_WIDGET_UNSET_FLAGS (event_box, GTK_NO_WINDOW);
 		gnome_popup_menu_attach (popup_menu, event_box, NULL);
+		GTK_WIDGET_SET_FLAGS (event_box, GTK_NO_WINDOW);
 
-		gtk_container_add (GTK_CONTAINER (event_box), hbox);
+		/* setup the tooltip */
+		set_tooltip (tooltips, event_box, child);
+		
+		/* show all children of our newly-created tab pane */
+		gtk_widget_show_all (hbox);
 
-		gtk_widget_show_all (event_box);
-		ret = event_box;
+		ret = hbox;
 	}
 
-	gtk_tooltips_set_tip (tooltips, ret, uri, NULL);
-
 	g_free (name);
-	g_free (uri);
 
 	return ret;
 }
